@@ -4,34 +4,34 @@ use std::path::{Path, PathBuf};
 use object::{Architecture, LittleEndian, Object, ObjectSection, ObjectSymbol, pod, RelocationEncoding, RelocationKind, SectionKind};
 use object::coff::CoffFile;
 use object::pe::{IMAGE_RESOURCE_DATA_IS_DIRECTORY, ImageResourceDataEntry, ImageResourceDirectory, ImageResourceDirectoryEntry};
-use object::read::archive::{ArchiveFile, ArchiveKind};
 use tempfile::{TempDir, tempdir};
 
+use crate::embed::coff::MachineType;
 use crate::embed::{create_object_file, link_manifest_gnu, link_manifest_msvc, TargetOs};
 use crate::new_manifest;
 
 #[test]
-fn create_lib() {
-    let res = do_embed_file(Architecture::X86_64, TargetOs::WindowsGnu);
-    let data = fs::read(&res.lib()).unwrap();
-    let archive = ArchiveFile::parse(&data[..]).unwrap();
-    assert_eq!(archive.kind(), ArchiveKind::Gnu);
-    assert_eq!(archive.members().map(|m| m.unwrap().name()).collect::<Vec<&[u8]>>(), &[b"manifest.o"]);
+fn create_obj() {
+    let res = do_embed_file(MachineType::X86_64, TargetOs::WindowsGnu);
+    let data = fs::read(&res.object_file()).unwrap();
+    let obj = CoffFile::parse(&data[..]).unwrap();
+    assert_eq!(obj.architecture(), Architecture::X86_64);
+    let expected_manifest = fs::read(&sample_manifest_path()).unwrap();
+    check_object_file(obj, &expected_manifest);
 }
 
 #[test]
 fn link_lib_gnu() {
-    let res = do_embed_file(Architecture::X86_64, TargetOs::WindowsGnu);
-    assert!(res.lib().exists());
-    let mut search_option = String::from("cargo:rustc-link-search=native=");
-    search_option.push_str(res.out_dir.path().to_str().unwrap());
-    assert_eq!(res.lines(), &[search_option.as_str(), "cargo:rustc-link-lib=static=embed-manifest"]);
+    let res = do_embed_file(MachineType::X86_64, TargetOs::WindowsGnu);
+    assert!(res.object_file().exists());
+    let object_option = format!("cargo:rustc-link-arg-bins={}", res.object_file().display());
+    assert_eq!(res.lines(), &[object_option.as_str()]);
 }
 
 #[test]
 fn link_file_msvc() {
-    let res = do_embed_file(Architecture::X86_64, TargetOs::WindowsMsvc);
-    assert!(!res.lib().exists());
+    let res = do_embed_file(MachineType::X86_64, TargetOs::WindowsMsvc);
+    assert!(!res.object_file().exists());
     let mut input_option = String::from("cargo:rustc-link-arg-bins=/MANIFESTINPUT:");
     input_option.push_str(res.manifest_path.canonicalize().unwrap().to_str().unwrap());
     assert_eq!(res.lines(), &[
@@ -48,8 +48,8 @@ struct EmbedResult {
 }
 
 impl EmbedResult {
-    fn lib(&self) -> PathBuf {
-        self.out_dir.path().join("libembed-manifest.a")
+    fn object_file(&self) -> PathBuf {
+        self.out_dir.path().join("embed-manifest.o")
     }
 
     fn lines(&self) -> Vec<&str> {
@@ -57,8 +57,12 @@ impl EmbedResult {
     }
 }
 
-fn do_embed_file(arch: Architecture, os: TargetOs) -> EmbedResult {
-    let manifest_path = Path::new(env!("CARGO_MANIFEST_DIR")).join("sample.exe.manifest");
+fn sample_manifest_path() -> PathBuf {
+    Path::new(env!("CARGO_MANIFEST_DIR")).join("sample.exe.manifest")
+}
+
+fn do_embed_file(arch: MachineType, os: TargetOs) -> EmbedResult {
+    let manifest_path = sample_manifest_path();
     let out_dir = tempdir().unwrap();
     let mut buf: Vec<u8> = Vec::new();
     if matches!(os, TargetOs::WindowsMsvc) {
@@ -72,7 +76,7 @@ fn do_embed_file(arch: Architecture, os: TargetOs) -> EmbedResult {
 #[test]
 fn object_file_x86() {
     let manifest = new_manifest("Test.X86").to_string().into_bytes();
-    let file = create_object_file(&manifest, Architecture::I386).write().unwrap();
+    let file = create_object_file(&manifest, MachineType::I386).unwrap();
     let obj = CoffFile::parse(&file[..]).unwrap();
     assert_eq!(obj.architecture(), Architecture::I386);
     check_object_file(obj, &manifest);
@@ -81,17 +85,16 @@ fn object_file_x86() {
 #[test]
 fn object_file_x86_64() {
     let manifest = new_manifest("Test.X86_64").to_string().into_bytes();
-    let file = create_object_file(&manifest, Architecture::X86_64).write().unwrap();
+    let file = create_object_file(&manifest, MachineType::X86_64).unwrap();
     let obj = CoffFile::parse(&file[..]).unwrap();
     assert_eq!(obj.architecture(), Architecture::X86_64);
     check_object_file(obj, &manifest);
 }
 
 #[test]
-#[ignore]
 fn object_file_aarch64() {
     let manifest = new_manifest("Test.AARCH64").to_string().into_bytes();
-    let file = create_object_file(&manifest, Architecture::Aarch64).write().unwrap();
+    let file = create_object_file(&manifest, MachineType::Aarch64).unwrap();
     let obj = CoffFile::parse(&file[..]).unwrap();
     assert_eq!(obj.architecture(), Architecture::Aarch64);
     check_object_file(obj, &manifest);
@@ -114,7 +117,11 @@ fn check_object_file(obj: CoffFile, expected_manifest: &[u8]) {
 
     // The data RVA in the resource data entry must be relocatable.
     let (addr, reloc) = rsrc01.relocations().next().unwrap();
-    assert_eq!(reloc.kind(), RelocationKind::ImageOffset);
+    if obj.architecture() == Architecture::Aarch64 {
+        assert_eq!(reloc.kind(), RelocationKind::Coff(2));
+    } else {
+        assert_eq!(reloc.kind(), RelocationKind::ImageOffset);
+    }
     assert_eq!(reloc.encoding(), RelocationEncoding::Generic);
     assert_eq!(addr, 0x48); // size of the directory table, three directories, and no strings
     assert_eq!(reloc.addend(), 0);
